//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     tlist.h
// Purpose:    Lists of Terms
// Last Change:27.09.2001
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2000-2001 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	TLIST_INCLUDED
#define TLIST_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef TERM_INCLUDED
#include "term.h"
#endif

//=============================================================================
// Private Constants:
//=============================================================================

//-----------------------------------------------------------------------------
// TLIST_MAGIC: Magic number (identifies objects of this class for debugging).
//-----------------------------------------------------------------------------

#ifdef VER_DEBUG
static const long TLIST_MAGIC =	0x544C4953L;	// 'TLIS'
#endif

//=============================================================================
// Class for Lists of Terms:
//=============================================================================

class tlist_c {

	public:

//-----------------------------------------------------------------------------
// Methods:
//-----------------------------------------------------------------------------

	// Constructor:
	tlist_c(term_t head);

	// Destructor:
	~tlist_c(void);

	// Head of the list:
	inline term_t	head(void) const
		{ CHECK(valid(), "tlist_c::head: Invalid Object");
		  return(Head); }

	// Tail of the list:
	inline tlist_c*	tail(void) const
		{ CHECK(valid(), "tlist_c::tail: Invalid Object");
		  return(Tail); }

	// Append a list (only possible if tail empty):
	void append(tlist_c* tail);

	// Print this term list:
	void print(void) const;

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	// Integrity Check:
	public:
	bool_t	valid(void) const;

	// Magic number (identifies objects of this class for debugging).
	private:
	long	Magic;		// Must be "TLIST_MAGIC"
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// Head: First term in the list.
	term_t	Head;

	// Tail: Rest of the list (i.e. this list except the first term).
	tlist_c *Tail;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//-----------------------------------------------------------------------------
// Define pointer type:
//-----------------------------------------------------------------------------

typedef tlist_c *tlist_t;

//=============================================================================
// End of Include File:
//=============================================================================

#endif

